@@ -42,14 +42,14 @@
 
 namespace PDepend;
 
+use ArrayIterator;
+use GlobIterator;
 use PDepend\Input\CompositeFilter;
 use PDepend\Input\Filter;
 use PDepend\Input\Iterator;
 use PDepend\Metrics\AnalyzerCacheAware;
-use PDepend\Metrics\AnalyzerClassFileSystemLocator;
 use PDepend\Metrics\AnalyzerFactory;
 use PDepend\Metrics\AnalyzerFilterAware;
-use PDepend\Metrics\AnalyzerLoader;
 use PDepend\Report\CodeAwareGenerator;
 use PDepend\Source\AST\ASTArtifactList\ArtifactFilter;
 use PDepend\Source\AST\ASTArtifactList\CollectionArtifactFilter;
@@ -62,6 +62,7 @@ use PDepend\Source\Language\PHP\PHPTokenizerInternal;
 use PDepend\Source\Tokenizer\Tokenizer;
 use PDepend\Util\Cache\CacheFactory;
 use PDepend\Util\Configuration;
+use SplFileObject;
 
 /**
  * PDepend analyzes php class files and generates metrics.
@@ -92,17 +93,24 @@ class Engine
      */
     protected $configuration = null;
 
+    /**
+     * Prefix for PHP streams.
+     *
+     * @var string
+     */
+    protected $phpStreamPrefix = 'php://';
+
     /**
      * List of source directories.
      *
-     * @var array(string)
+     * @var array<string>
      */
     private $directories = array();
 
     /**
      * List of source code file names.
      *
-     * @var array(string)
+     * @var array<string>
      */
     private $files = array();
 
@@ -158,7 +166,7 @@ class Engine
     /**
      * List of analyzer options.
      *
-     * @var array(string=>mixed)
+     * @var array<string, mixed>
      */
     private $options = array();
 
@@ -214,7 +222,7 @@ class Engine
     {
         $dir = realpath($directory);
 
-        if (!is_dir($dir)) {
+        if ($dir === false || !is_dir($dir)) {
             throw new \InvalidArgumentException("Invalid directory '{$directory}' added.");
         }
 
@@ -229,17 +237,21 @@ class Engine
      */
     public function addFile($file)
     {
-        if ($file === '-' || $file === 'php://stdin') {
-            $fileName = 'php://stdin';
-        } else {
-            $fileName = realpath($file);
-
-            if (!is_file($fileName)) {
-                throw new \InvalidArgumentException(sprintf('The given file "%s" does not exist.', $file));
-            }
+        if ($file === '-') {
+            $file = $this->phpStreamPrefix . 'stdin';
         }
 
-        $this->files[] = $fileName;
+        if ($this->isPhpStream($file)) {
+            $this->files[] = $file;
+
+            return;
+        }
+
+        if (!is_file($file)) {
+            throw new \InvalidArgumentException(sprintf('The given file "%s" does not exist.', $file));
+        }
+
+        $this->files[] = realpath($file);
     }
 
     /**
@@ -279,7 +291,7 @@ class Engine
     /**
      * Sets analyzer options.
      *
-     * @param  array(string=>mixed) $options The analyzer options.
+     * @param  array<string, mixed> $options The analyzer options.
      * @return void
      */
     public function setOptions(array $options = array())
@@ -364,7 +376,7 @@ class Engine
 
         $classes = 0;
         foreach ($this->namespaces as $namespace) {
-            $classes += $namespace->getTypes()->count();
+            $classes += count($namespace->getTypes());
         }
         return $classes;
     }
@@ -635,7 +647,11 @@ class Engine
         $fileIterator = new \AppendIterator();
 
         foreach ($this->files as $file) {
-            $fileIterator->append(new Iterator(new \GlobIterator($file), $this->fileFilter));
+            $fileIterator->append(
+                $this->isPhpStream($file)
+                    ? new ArrayIterator(array(new SplFileObject($file)))
+                    : new Iterator(new GlobIterator($file), $this->fileFilter)
+            );
         }
 
         foreach ($this->directories as $directory) {
@@ -661,7 +677,7 @@ class Engine
             if (is_string($file)) {
                 $files[$file] = $file;
             } else {
-                $pathname         = realpath($file->getPathname());
+                $pathname = $file->getRealPath() ?: $file->getPathname();
                 $files[$pathname] = $pathname;
             }
         }
@@ -675,7 +691,7 @@ class Engine
         ksort($files);
         // END
 
-        return new \ArrayIterator(array_values($files));
+        return new ArrayIterator(array_values($files));
     }
 
     private function createAnalyzers($options)
@@ -702,4 +718,9 @@ class Engine
 
         return $analyzers;
     }
+
+    private function isPhpStream($path)
+    {
+        return substr($path, 0, strlen($this->phpStreamPrefix)) === $this->phpStreamPrefix;
+    }
 }
