@@ -4,7 +4,7 @@
  *
  * PHP Version 5
  *
- * Copyright (c) 2008-2015, Manuel Pichler <mapi@pdepend.org>.
+ * Copyright (c) 2008-2017 Manuel Pichler <mapi@pdepend.org>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,7 @@
  * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  *
- * @copyright 2008-2015 Manuel Pichler. All rights reserved.
+ * @copyright 2008-2017 Manuel Pichler. All rights reserved.
  * @license http://www.opensource.org/licenses/bsd-license.php BSD License
  */
 
@@ -60,7 +60,7 @@ use PDepend\Source\ASTVisitor\ASTVisitor;
  * }
  * </code>
  *
- * @copyright 2008-2015 Manuel Pichler. All rights reserved.
+ * @copyright 2008-2017 Manuel Pichler. All rights reserved.
  * @license http://www.opensource.org/licenses/bsd-license.php BSD License
  */
 class ASTParameter extends AbstractASTArtifact
@@ -89,7 +89,7 @@ class ASTParameter extends AbstractASTArtifact
     /**
      * The wrapped formal parameter instance.
      *
-     * @var \PDepend\Source\AST\ASTFormalParameter
+     * @var ASTFormalParameter
      */
     private $formalParameter = null;
 
@@ -103,9 +103,9 @@ class ASTParameter extends AbstractASTArtifact
     /**
      * Constructs a new parameter instance for the given AST node.
      *
-     * @param \PDepend\Source\AST\ASTFormalParameter $formalParameter
+     * @param ASTFormalParameter $formalParameter
      */
-    public function __construct(\PDepend\Source\AST\ASTFormalParameter $formalParameter)
+    public function __construct(ASTFormalParameter $formalParameter)
     {
         $this->formalParameter    = $formalParameter;
         $this->variableDeclarator = $formalParameter->getFirstChildOfType(
@@ -172,7 +172,7 @@ class ASTParameter extends AbstractASTArtifact
      * This method will return the class where the parent method was declared.
      * The returned value will be <b>null</b> if the parent is a function.
      *
-     * @return \PDepend\Source\AST\AbstractASTClassOrInterface
+     * @return \PDepend\Source\AST\AbstractASTClassOrInterface|null
      * @since  0.9.5
      */
     public function getDeclaringClass()
@@ -210,7 +210,7 @@ class ASTParameter extends AbstractASTArtifact
      * Returns the class type of this parameter. This method will return
      * <b>null</b> for all scalar type, only classes or interfaces are used.
      *
-     * @return \PDepend\Source\AST\AbstractASTClassOrInterface
+     * @return \PDepend\Source\AST\AbstractASTClassOrInterface|null
      * @since  0.9.5
      */
     public function getClass()
@@ -327,10 +327,8 @@ class ASTParameter extends AbstractASTArtifact
     public function getDefaultValue()
     {
         $value = $this->variableDeclarator->getValue();
-        if ($value === null) {
-            return null;
-        }
-        return $value->getValue();
+
+        return $value === null ? null : $value->getValue();
     }
 
     /**
