@@ -43,10 +43,6 @@
 
 namespace PDepend\Source\Language\PHP;
 
-use PDepend\Source\AST\ASTValue;
-use PDepend\Source\Parser\TokenStreamEndException;
-use PDepend\Source\Parser\UnexpectedTokenException;
-use PDepend\Source\Tokenizer\Tokenizer;
 use PDepend\Source\Tokenizer\Tokens;
 
 /**
@@ -58,13 +54,13 @@ use PDepend\Source\Tokenizer\Tokens;
  * @license http://www.opensource.org/licenses/bsd-license.php BSD License
  * @since 0.9.20
  */
-class PHPParserGeneric extends PHPParserVersion71
+class PHPParserGeneric extends PHPParserVersion74
 {
     /**
      * Tests if the given token type is a reserved keyword in the supported PHP
      * version.
      *
-     * @param  $tokenType
+     * @param  integer $tokenType
      * @return boolean
      * @since  1.1.1
      */
@@ -141,29 +137,6 @@ class PHPParserGeneric extends PHPParserVersion71
         return false;
     }
 
-    /**
-     * Implements some quirks and hacks to support php here- and now-doc for
-     * PHP 5.2.x versions :/
-     *
-     * @return \PDepend\Source\AST\ASTHeredoc
-     * @since 1.0.0
-     */
-    protected function parseHeredoc()
-    {
-        $heredoc = parent::parseHeredoc();
-        if (version_compare(phpversion(), "5.3.0alpha") >= 0) {
-            return $heredoc;
-        }
-
-        // Consume dangling semicolon
-        $this->tokenizer->next();
-
-        $token = $this->tokenizer->next();
-        preg_match('(/\*(\'|")\*/)', $token->image, $match);
-
-        return $heredoc;
-    }
-
     /**
      * Parses additional static values that are valid in the supported php version.
      *
